@@ -7,11 +7,14 @@ Amazon::S3::Thin::Credentials - AWS credentials data container
 =head1 SYNOPSIS
 
     my $credentials = Amazon::S3::Thin::Credentials->new(
-        $aws_access_key_id, $aws_secret_access_key
+        $aws_access_key_id, $aws_secret_access_key,
+        # optional:
+        $aws_session_token
     );
     
     my $key = $credentials->access_key_id();
     my $secret = $credentials->secret_access_key();
+    my $session_token = $credentials->session_token();
 
 1;
 
@@ -25,10 +28,11 @@ use strict;
 use warnings;
 
 sub new {
-    my ($class, $key, $secret) = @_;
+    my ($class, $key, $secret, $session_token) = @_;
     my $self = {
         key => $key,
         secret => $secret,
+        session_token => $session_token,
     };
     return bless $self, $class;
 }
@@ -55,4 +59,15 @@ sub secret_access_key {
     return $self->{secret};
 }
 
+=head2 session_token()
+
+Returns session_token
+
+=cut
+
+sub session_token {
+    my $self = shift;
+    return $self->{session_token};
+}
+
 1;
